@@ -160,15 +160,15 @@
    (Mikron's European Patent EP 0473569 A2 was filed 23 August 1991, with a
    priority date of 23 Aug 1990.)
    Mikron was subsequently acquired by Philips Semiconductors in 1995.
-   Philips Semiconductors divsion subsequently became NXP.
+   Philips Semiconductors division subsequently became NXP.
 
    + Modulation read/write device -> transponder: 100 % ASK and binary pulse
      length coding
    + Modulation transponder -> read/write device: Strong ASK modulation,
      selectable Manchester or Biphase coding
-   + Hitag S, Hitag u; anti-collision procedure
+   + Hitag S, Hitag µ; anti-collision procedure
    + Fast anti-collision protocol
-   + Hitag u; optional Cyclic Redundancy Check (CRC)
+   + Hitag µ; optional Cyclic Redundancy Check (CRC)
    + Reader Talks First mode
    + Hitag 2 & later; Transponder Talks First (TTF) mode
    + Temporary switch from Transponder Talks First into Reader Talks First
@@ -177,7 +177,7 @@
    + Data rates transponder to read/write device: 2 kbit/s, 4 kbit/s, 8 kbit/s
    + 32-bit password feature
    + Hitag 2, S = 32-bit Unique Identifier
-   + Hitag u = 48-bit Unique Identifier
+   + Hitag µ = 48-bit Unique Identifier
    + Selectable password modes for reader / tag mutual authentication
      (Hitag 1 has 2 pairs of keys, later versions have 1 pair)
    + Hitag 2 & Hitag S; Selectable encrypted mode, 48 bit key
@@ -188,16 +188,16 @@
 
    HITAG 2            256 Bit total memory        Read/Write
                       8 pages of 32 bits, inc UID (32),
-		      secret key (64), password (24), config (8)
+              secret key (64), password (24), config (8)
 
    HITAG S 32         32 bits Unique Identifier   Read Only
    HITAG S 256        256 bits total memory       Read/Write
    HITAG S 2048       2048 bits total memory      Read/Write
 
-   HITAG u RO64       64 bits total memory        Read Only
-   HITAG u            128 bits total memory       Read/Write
-   HITAG u Advanced   512 bits total memory       Read/Write
-   HITAG u Advanced+  1760 bits total memory      Read/Write
+   HITAG µ RO64       64 bits total memory        Read Only
+   HITAG µ            128 bits total memory       Read/Write
+   HITAG µ Advanced   512 bits total memory       Read/Write
+   HITAG µ Advanced+  1760 bits total memory      Read/Write
 
    Default 48-bit key for Hitag 2, S encryption:
        "MIKRON"       =  O  N  M  I  K  R
@@ -208,7 +208,9 @@
 
 // We want the crypto functions to be as fast as possible, so optimize!
 // The best compiler optimization in Microchip's free XC32 edition is -O1
+#if defined(__GNUC__) && !defined(__clang__)
 #pragma GCC optimize("O1")
+#endif
 
 // private, nonlinear function to generate 1 crypto bit
 static uint32_t hitag2_crypt(uint64_t x);
@@ -227,9 +229,9 @@ static uint32_t hitag2_crypt(uint64_t x);
 
 
 static uint32_t hitag2_crypt(uint64_t x) {
-    const uint32_t ht2_function4a = 0x2C79;	// 0010 1100 0111 1001
-    const uint32_t ht2_function4b = 0x6671;	// 0110 0110 0111 0001
-    const uint32_t ht2_function5c = 0x7907287B;	// 0111 1001 0000 0111 0010 1000 0111 1011
+    const uint32_t ht2_function4a = 0x2C79; // 0010 1100 0111 1001
+    const uint32_t ht2_function4b = 0x6671; // 0110 0110 0111 0001
+    const uint32_t ht2_function5c = 0x7907287B; // 0111 1001 0000 0111 0010 1000 0111 1011
     uint32_t bitindex;
 
     bitindex = (ht2_function4a >> pickbits2_2(x, 1, 4)) & 1;
@@ -367,4 +369,6 @@ uint32_t hitag2_nstep(Hitag_State *pstate, uint32_t steps) {
 }
 
 // end of crypto core, revert to default optimization level
+#if defined(__GNUC__) && !defined(__clang__)
 #pragma GCC reset_options
+#endif
